<?php
// $Id: xc_search_date_facet_properties.php,v 1.1.2.10 2010/06/01 15:03:43 pkiraly Exp $

function xc_search_date_facet_properties_load($set_id) {
  $sql = "SELECT * FROM {xc_search_date_facet_properties} WHERE set_id = %d";
  $result = db_query(db_rewrite_sql($sql, '{xc_search_date_facet_properties}', 
    'set_id'), $set_id);
  $data = db_fetch_object($result);
  return $data;
} 

function xc_search_date_facet_properties_fieldset() {
  return array(
    'name' => t('Name'),
    'description' => t('Description'),
    'start' => t('Start'),
    'end' => t('End'),
    'gap' => t('Gap'),
    'hardend' => t('Hardend'),
    'other' => t('Other'),
  );
}

function xc_search_date_facet_properties_options() {
  global $XC_SEARCH_GLOBAL_OPTIONS;

  return array(
    'other'  => array(  
      'before'  => t('before'),
      'after'   => t('after'),
      'between' => t('between'),
      'none'    => t('none'),
      'all'     => t('all'),
    ),
    'hardend' => $XC_SEARCH_GLOBAL_OPTIONS['true_false'],
  );
}

function xc_search_date_facet_properties_view($record) {
  return xc_util_view($record, 
    xc_search_date_facet_properties_fieldset(), 
    xc_search_date_facet_properties_options()
  );
}

function xc_search_date_facet_properties_list() {
  
  $headers = array(t('Name'), t('Description'));
  $rows = array();
  
  $sql = "SELECT set_id, name, description FROM {xc_search_date_facet_properties}";
  $result = db_query(db_rewrite_sql($sql, '{xc_search_date_facet_properties}', 
    'set_id'));
  while ($data = db_fetch_object($result)) {
    $rows[] = array(l($data->name, 'xc/search/displaying/facet/datefacetset/'.
      $data->set_id), $data->description);
  }
  return theme('table', $headers, $rows);
}

/**
 * Add a new server
 * @return string The add server page with a form
 */
function xc_search_date_facet_properties_add() {
  drupal_set_title('Facet attribute set');
  return drupal_get_form('xc_search_date_facet_properties_form');
}

function xc_search_date_facet_properties_form() {
  $schema = drupal_get_schema_unprocessed('xc_search', 'xc_search_date_facet_properties');
  drupal_set_title($schema['description']);
  $options = array(
    'omit' => array('set_id'),
    'select' => xc_search_date_facet_properties_options(),
    'label' => xc_search_date_facet_properties_fieldset(),
  );
  $form = xc_util_build_autoform($schema, $options);
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );

  return $form;
}

function xc_search_date_facet_properties_edit_form(&$form_state, $set) {
  $form = xc_search_date_facet_properties_form();
  $form['set_id'] = array(
    '#type'  => 'hidden',
    '#value' => $set->set_id,
  );
  
  foreach (xc_search_date_facet_properties_fieldset() as $name => $label) {
    $form[$name]['#default_value'] = $set->$name;
  }
  
  $form['submit']['#value']      = t('Save');
  $form['delete'] = array(
    '#type' => 'submit',
    '#value' => t('Delete'),
  );

  return $form;
} 

function xc_search_date_facet_properties_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $record = new stdClass();
  foreach (xc_search_date_facet_properties_fieldset() as $name => $label) {
    $record->$name = $values[$name];
  }

  $ret_val = drupal_write_record('xc_search_date_facet_properties', $record);
  if ($ret_val == SAVED_NEW) {
    drupal_set_message(t('Thank you for adding date facet settings "%name"',
      array('%name' => $record->name)));
    $form_state['redirect'] = 'xc/search/displaying/facet/datefacetset/' 
      . $record->set_id;
  }
  else {
    drupal_set_message(t('Unexpected error. Failed to create new facet settings.'));
  }
}

function xc_search_date_facet_properties_edit_form_submit($form, &$form_state) {
  $values = $form_state['clicked_button']['#post'];
  $record = new stdClass();
  $record->set_id = $values['set_id'];
  $record->name   = $values['name'];

  if ($form_state['clicked_button']['#value'] == t('Delete')) {
    $sql = 'DELETE FROM {xc_search_date_facet_properties} WHERE set_id = %d';
    $result = db_query($sql, $record->set_id);

    if ($result == SAVED_DELETED) { // repository is deleted
      drupal_set_message(t('%name removed', array('%name' => $record->name)));
      $form_state['redirect'] = 'xc/search/displaying/facet/datefacetset/list';
    }
    else {
      drupal_set_message(t('Unexpected error. Failed to remove repository.'));
    }
  }
  elseif ($form_state['clicked_button']['#value'] == t('Save')) {
    foreach (xc_search_date_facet_properties_fieldset() as $name => $label) {
      $record->$name = $values[$name];
    }
   
    $result = drupal_write_record('xc_search_date_facet_properties', $record, 
      'set_id');
    if ($result == SAVED_UPDATED) { // repository is updated
      drupal_set_message(t('%name modified', array('%name' => $record->name)));
      $form_state['redirect'] = 'xc/search/displaying/facet/datefacetset/' . 
        $record->set_id . '/view';
    }
    else {
      drupal_set_message(t('Unexpected error. Failed to modify facet attribute set.'));
    }
  }
  else {
    drupal_set_message(t('Unexpected error. Something else happened.'));
  }
}

function xc_search_date_facet_properties_default() {
  $raw_values = <<<END
name;description;start;end;gap;hardend;other
100 years;100 years;1000-01-01T00:00:01Z;NOW;+100YEARS;1;all
END;

  return $raw_values;
}

/**
 * Convert the properties to array of Solr parameters
 * @param $params (Object)
 *   The properties record
 * @param $facet_name (String)
 *   The name of the facet (default: NULL).
 * @return (Array)
 *   The array of Solr parameters
 */
function xc_search_date_facet_properties2solr_params($params, $facet_name = NULL) {
  
  $prefix = is_null($facet_name) ? '' : 'f.' . $facet_name . '.facet.';
   
  return array(
    'facet.date'             => $facet_name, // TODO: not sure about this
    $prefix . 'date.start'   => $params->start,
    $prefix . 'date.end'     => $params->end,
    $prefix . 'date.gap'     => $params->gap,
    $prefix . 'date.hardend' => $params->hardend ? 'true' : 'false',
    $prefix . 'date.other'   => $params->other,
  );
}
