<?php
// $Id: xc_search_facet_group.php,v 1.1.2.8 2010/06/01 15:03:43 pkiraly Exp $

function xc_search_facet_group_load($facet_id) {
  $sql = "SELECT * FROM {xc_search_facet_group} WHERE facet_id = %d";
  $result = db_query(db_rewrite_sql($sql, '{xc_search_facet_group}', 'facet_id'), 
    $facet_id);
  $data = db_fetch_object($result);
  return $data;
} 

function xc_search_facet_group_title($facet) {
  return t('@name facet group', array('@name' => $facet->name));
}

function xc_search_facet_group_fieldset() {
  return array(
    'name'             => t('Name'),
    'description'      => t('Description'),
    'metadata_schema'  => t('Metadata schema'),
    'attribute_set_id' => t('Default attribute set'),
  );
}

function xc_search_facet_group_view($facet) {
  
  $fields = xc_search_facets_by_group($facet->facet_id);
  $headers = array(t('Property'), t('Value'));
  $rows = array();
  $labels = xc_search_facet_group_fieldset();
  
  $attribute_set = xc_search_field_facet_properties_load($facet->attribute_set_id);
  $rows[] = array(
    $labels['name'], 
    $facet->name
  );
  $rows[] = array(
    $labels['description'], 
    $facet->description
  );
  $rows[] = array(
    $labels['metadata_schema'], 
    l($facet->metadata_schema, 'xc/metadata/formats/' . $facet->metadata_schema)
  );
  $rows[] = array(
    $labels['attribute_set_id'], 
    l($attribute_set->name, 'xc/search/displaying/facet/fieldfacetset/' 
      . $facet->attribute_set_id)
  );
  
  $output = theme('table', array(t('Property'), t('Value')), $rows);

  if (count($fields) == 0) {
    $output .= t('No field connected to this facet group.');
    $output .= ' ';
    $output .= l(t('Please add field!'), 'xc/search/displaying/facet/group/' 
      . $facet->facet_id . '/add_field_facet');
  } 
  else {
    $rows = array();
    foreach ($fields as $field) {
      $attribute_set = xc_search_get_attribute_set($field->type, 
        $field->attribute_set_id);
      $type_path = ($field->type == 'field') ? '' : $field->type;
      $rows[] = array($field->label, $field->name, $field->type, 
        l($attribute_set->name, 
          'xc/search/displaying/facet/'. $field->type . 'facetset/'
          . $field->attribute_set_id . '/view'),
        l(t('edit'), 'xc/search/displaying/facet/' . $type_path . 'field/' 
          . $field->field_id . '/edit') . ', ' .
        l(t('remove'), 'xc/search/displaying/facet/field/' . $field->field_id 
          . '/remove'),
      );
    }
    $output .= t('fields');
    $output .= theme(
      'table', 
      array(t('Label'), t('Name'), t('Type'), t('Attribute set'), t('Actions')), 
      $rows
    );
    $output .= l(t('Reorder facets'),
      'xc/search/displaying/facet/group/' . $facet->facet_id . '/reorder');
  }
  
  return $output;
}

function xc_search_facet_group_reorder_form(&$form_state, $facet) {
  $fields = xc_search_facets_by_group($facet->facet_id);
  if (count($fields) == 0) {
    return;
  }
  $rows = array();
  $form = array(
    '#tree' => TRUE,
  );

  $weight_delta = count($fields);
  foreach ($fields as $field) {
    $key = $field->field_id;
    $form['facets'][$key] = array();
    $attribute_set = xc_search_get_attribute_set($field->type, 
      $field->attribute_set_id);
    $type_path = ($field->type == 'field') ? '' : $field->type;
    
    $form['facets'][$key]['label'] = array('#value' => $field->label);
    $form['facets'][$key]['name'] = array('#value' => $field->name);
    $form['facets'][$key]['type'] = array('#value' => $field->type);
    $form['facets'][$key]['weight'] = array(
      '#type' => 'weight',
      '#default_value' => $field->weight,
      '#delta' => $weight_delta,
    );
    $form['facets'][$key]['attributes'] = array('#value' => l(
      $attribute_set->name,
      'xc/search/displaying/facet/'. $field->type . 'facetset/' 
      . $field->attribute_set_id . '/view') 
    );
  }
  $form['facet_id'] = array(
    '#type' => 'hidden',
    '#value' => $facet->facet_id,
  );
  
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save facets\' order'),
  );

  return $form;
}

/**
 * Save the order of facets
 * @param $form
 * @param $form_state
 */
function xc_search_facet_group_reorder_form_submit($form, &$form_state) {
  $id = $form_state['values']['facet_id'];
  $facets = $form_state['values']['facets'];
  foreach($facets as $key => $facet) {
    db_query("UPDATE {xc_search_facet_field} SET weight = %d WHERE name = '%s'
      AND facet_id = %d",
       $facet['weight'],  $form['facets'][$key]['name']['#value'], $id);
  }
  drupal_set_message(t('Order of facets has been successfully updated.'));
  $form_state['redirect'] = 'xc/search/displaying/facet/group/' . $id . '/view';
}

/**
 * Theming the facet reordering form
 * @param $form
 * @return unknown_type
 */
function theme_xc_search_facet_group_reorder_form($form) {
  //return t('hello');
  $rows = array();
  drupal_add_tabledrag('facet-sort', 'order', 'sibling', 'sort');
  
  foreach(element_children($form['facets']) as $key) {
    $facet = &$form['facets'][$key];
    $facet['weight']['#attributes']['class'] = 'sort';
    
    $cells = array(
      drupal_render($facet['label']),
      drupal_render($facet['name']),
      drupal_render($facet['type']),
      drupal_render($facet['attributes']),
      drupal_render($facet['weight'])
    );
      
    
    $rows[] = array(
      'data' => $cells, 
      'class' => 'draggable'
    ); 
  }
  $output = theme('table', 
    array(t('Label'), t('Name'), t('Type'), t('Attribute set'), ''), 
    $rows, 
    array('id' => 'facet-sort')
  );
  $output .= drupal_render($form);
  return $output;
}

function xc_search_facet_group_list() {
  $headers = array(t('Name'), t('Description'));
  $rows = array();
  
  $sql = 'SELECT * FROM {xc_search_facet_group}';
  $result = db_query(db_rewrite_sql($sql, '{xc_search_facet_group}', 'facet_id'));
  while ($data = db_fetch_object($result)) {
    $rows[] = array(
      l($data->name, 'xc/search/displaying/facet/group/' . $data->facet_id), 
      $data->description
    );
  }
  return theme('table', $headers, $rows);
}

function xc_search_facet_group_list_values() {
  $sql = 'SELECT * FROM {xc_search_facet_group}';
  $result = db_query(db_rewrite_sql($sql, '{xc_search_facet_group}', 'facet_id'));
  $rows = array();
  while ($data = db_fetch_object($result)) {
    $rows[$data->facet_id] = $data->name;
  }
  return $rows;
}

function xc_search_facet_group_add() {
  drupal_set_title('Create a new facet navigation');
  return drupal_get_form('xc_search_facet_group_form');
}

function xc_search_facet_group_form() {
  $schema = drupal_get_schema_unprocessed('xc_search', 'xc_search_facet_group');
  drupal_set_title($schema['description']);
  $options = array(
    'omit' => array('facet_id'),
    'hidden' => array(),
    'label' => xc_search_facet_group_fieldset(),
    'select' => array(
      'metadata_schema' => xc_format_get_schemas(),
      'attribute_set_id' => array(),
    ),
  );
  // attribute sets
  $sql = "SELECT set_id, name, description FROM {xc_search_field_facet_properties}";
  $result = db_query(db_rewrite_sql($sql, '{xc_search_field_facet_properties}',
    'set_id'));
  while ($data = db_fetch_object($result)) {
    $options['select']['attribute_set_id'][$data->set_id] =  $data->name;
  }
  
  $form = xc_util_build_autoform($schema, $options);
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

function xc_search_facet_group_edit_form(&$form_state, $facet) {
  
  $form = xc_search_facet_group_form();
  $form['facet_id'] = array(
    '#type'  => 'hidden',
    '#value' => $facet->facet_id,
  );
  $form['name']['#default_value']            = $facet->name;
  $form['description']['#default_value']     = $facet->description;
  $form['metadata_schema']['#default_value'] = $facet->metadata_schema;
  $form['attibute_set_id']['#default_value'] = $facet->attibute_set_id;
  
  $form['submit']['#value']      = t('Save');
  $form['delete'] = array(
    '#type' => 'submit',
    '#value' => t('Delete'),
  );

  return $form;
} 

function xc_search_facet_group_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $record = new stdClass();
  foreach (xc_search_facet_group_fieldset() as $name => $label) {
    $record->$name = $values[$name];
  }
  
  $ret_val = drupal_write_record('xc_search_facet_group', $record);
  if ($ret_val == SAVED_NEW) {
    drupal_set_message(t('Successfully added facet group "%name"',
      array('%name' => $record->name)));
    $form_state['redirect'] = 'xc/search/displaying/facet/group/' 
      . $record->facet_id . '/view';
  }
  else {
    drupal_set_message(t('Unexpected error. Failed to create new facet group.'));
  }
}

function xc_search_facet_group_edit_form_submit($form, &$form_state) {
  $values = $form_state['clicked_button']['#post'];
  $record = new stdClass();
  $record->facet_id      = $values['facet_id'];
  $record->name          = $values['name'];

  if ($form_state['clicked_button']['#value'] == t('Delete')) {
    $sql = 'DELETE FROM {xc_search_facet_group} WHERE set_id = %d';
    $result = db_query($sql, $record->set_id);
    // TODO: delete connected fields

    if ($result == SAVED_DELETED) { // repository is deleted
      drupal_set_message(t('%name removed', array('%name' => $record->name)));
      $form_state['redirect'] = 'xc/search/displaying/facet/list';
    }
    else {
      drupal_set_message(t('Unexpected error. Failed to remove facet group.'));
    }
  }
  elseif ($form_state['clicked_button']['#value'] == t('Save')) {
    foreach (xc_search_facet_group_fieldset() as $name => $label) {
      $record->$name = $values[$name];
    }
    $result = drupal_write_record('xc_search_facet_group', $record, 'facet_id');

    if ($result == SAVED_UPDATED) { // repository is updated
      drupal_set_message(t('%name modified', array('%name' => $record->name)));
      $form_state['redirect'] = 'xc/search/displaying/facet/group/' 
        . $record->facet_id . '/view';
    }
    else {
      drupal_set_message(t('Unexpected error. Failed to modify facet group.'));
    }
  }
  else {
    drupal_set_message(t('Unexpected error. Something else happened.'));
  }
} 

function xc_search_facet_group_default() {
  $raw_values = <<<END
name;description;metadata_schema;attribute_set_id
default;default facet group;xc;1
END;

  return $raw_values;
}