<?php
// $Id: xc_search_block.inc,v 1.1.2.3 2010/05/01 15:12:26 pkiraly Exp $

/**
 * @file
 * functions used for handlink blocks near the search result list, or the
 * full display
 */

function xc_search_block_facets($current_path) {
  global $_xc_search_server, $base_path;

  $search_path = $_GET['q'];
  
  $query = xc_search_current_query();
  $ui = xc_search_current_ui();
  if(empty($ui)) {
    $ui = xc_search_ui_get($_GET['search_ui']);
  }
  $is_prepopulated = FALSE;
  if (empty($query)) {
    if($ui->do_prepopulate_facets == 1) {
      $is_prepopulated = TRUE;
      $query = '*:*';
    }
    else {
      return;
    }
  }

  $blocks['subject'] = t('Faceted navigation');
  if ($query != '*:*' 
      && !preg_match('/^(xc__workExpressed_t|xc__expressionManifested_t|id|):/', 
          $query)) {
    // TODO: create a better display than raw Solr query
    
    $blocks['content'] = t('The query is: %query', array(
      '%query' => xc_search_display_query($query)
    ));
  }
  $facet_group = xc_search_facet_group_load($ui->facet_id);
  if(empty($facet_group)) {
    return;
  }
  $facet_objects = xc_search_facets_by_group($ui->facet_id);
  if(empty($facet_objects)) {
    return;
  }
  $default_field_params = xc_search_field_facet_properties_load(
    $facet_group->attribute_set_id
  );
      
  $params = array(
    'facet'          => 'true',
    'facet.limit'    => $default_field_params->limitation,
    'facet.mincount' => $default_field_params->mincount,
  );

  $type_map = array(
    'query' => 'facet_queries',
    'date'  => 'facet_dates',
    'field' => 'facet_fields',
  );

  $facets = array();
  foreach ($facet_objects as $facet) {
    $additional = xc_search_facet_load_by_name($facet->name);
    $field = $facet->name . $additional->type;
    $facets[$field] = array(
      'type' => $type_map[$facet->type],
      'field' => $field,
      'label' => t($additional->label)
    );
    if($facet->type == 'field') {
      $params['facet.field'][] = $field;
    }
    if(isset($facet->attribute_set_id) && $facet->attribute_set_id != 0) {
      $field_params = xc_search_get_attribute_set($facet->type, 
        $facet->attribute_set_id);
      $facet_params = xc_search_facet_properties2solr_params($facet->type, 
        $field_params, $facet->name . $additional->type);
      if($facet->type == 'date') {
        $facet_name = $facet->name . $additional->type;
        $current_gap = $_GET[$facet_name . '_gap'];
        if(!is_null($current_gap)) {
          $gap = xc_solr_parse_gap($current_gap);
          $param_name = 'f.' . $facet_name . '.facet.date.gap';
          if ($gap->quantity == 100 || $gap->quantity == 10) {
            $facet_params[$param_name] = '+' . ($gap->quantity / 10) 
              . $gap->measure;
          }
          else if ($gap->quantity == 1) {
            if ($gap->measure = 'YEAR') {
              $facet_params[$param_name] = '+1MONTH';
            }
            else if ($gap->measure = 'MONTH') {
              $facet_params[$param_name] = '+1DAY';
            }
          }
          foreach ( $_GET['filter'] as $filter) {
            if (preg_match('/' . $facet->name . $additional->type 
              . ':\[(.*?) TO (.*?)\]/', $filter, $matches)) {
              $facet_params['f.' . $facet_name . '.facet.date.start'] = $matches[1];
              $facet_params['f.' . $facet_name . '.facet.date.end'] = $matches[2];
              break;
            }
          }
        }
      }
      $params += $facet_params;
    }
  }
  
  $filters = empty($_GET['filter']) ? array() : $_GET['filter'];
  $search_type = !empty($_GET['search_type']) ? $_GET['search_type'] : 'standard';
  if ($search_type == 'standard') {
    if (!in_array('type:manifestation', $filters)) {
      $filters[] = 'type:manifestation';
    }
  }
  if (!empty($filters)) {
    $params['fq'] = $filters;
  }

  $base_link = $base_path . $current_path;
  $no_query_path = $base_link;
  $non_filter_params = array();
  if (strstr($current_path, '?')) {
    $no_query_path = preg_replace('/\?.*$/', '', $current_path);
    $no_filters_path = preg_replace('/\?.*$/', '?', $base_link);
    $appended = FALSE;
    foreach ($_GET as $key => $value) {
      if ($key != 'q' && $key != 'filter') {
        if ($appended == FALSE) {
          $appended = TRUE;
        }
        else {
          $no_filters_path .= '&';
        }
        $no_filters_path .= $key . '=' . $value;
        $non_filter_params[$key][] = $value;
      }
    }
  }
  if (!empty($filters)) {
    $filter_list = array();
    foreach ($filters as $filter) {
      if (preg_match('/^type:/', $filter)) {
        continue;
      }
      $current_non_filter_params = $non_filter_params;

      // creating remove link
      $other_filters = array_diff($filters, array($filter));

      // creating label
      list($facet_field, $facet_term) = split(':', $filter, 2);

      // field name to display
      $label = isset($facets[$facet_field])
        ? $facets[$facet_field]['label']
        : $facet_field;
      $label .= ': ';

      // get the value to display
      // field facet value
      if ($facets[$facet_field]['type'] == 'facet_fields') {
        $label .= preg_replace('/^"|"$/', '', $facet_term);
      }
      // date facet value
      elseif ($facets[$facet_field]['type'] == 'facet_dates') {
        unset($current_non_filter_params['date_df_gap']);
        $value = preg_replace('/^\[|\]$/', '', $facet_term);
        list($from, $to) = split(' TO ', $value);
        $from = substr($from, 0, 4);
        $to = substr($to, 0, 4);
        $label .= ($from != $to) ? $from . '-' . $to : $from;
      }
      // TODO: query facet value
      elseif ($facets[$facet_field]['type'] == 'facet_queries') {
        $value = preg_replace('/^\[|\]$/', '', $facet_term);
        $value_parts = split(' TO ', $value);
        $label .= join('-', $value_parts);
      }

      $filter_list[] = theme('xc_search_block_filter', 
         $search_path, $label, array_merge($current_non_filter_params, 
           array('filter' => $other_filters)));
    }
    if (!empty($filter_list)) {
      $blocks['content'] .= theme('item_list', $filter_list, 
        t('Active filters'), 'ul', array('class' => 'xc_active_filters'));
    }
  }

  $response = $_xc_search_server->search($query, 0, 0, $params, TRUE);

  // displaying each facets
  $options = array(
    'search_path'        => $search_path,
    'filters'            => $filters,
    'non_filter_params'  => $non_filter_params,
    'is_prepopulated'    => $is_prepopulated,
  );
  // $facet_objects
  foreach ($facets as $field => $facet) {
    $type  = $facet['type'];

    // get field facets
    if ($type == 'facet_fields' 
        && isset($response->facet_counts->$type)
        && isset($response->facet_counts->$type->$field)) {
      $links = xc_search_block_get_facet_fields($field, $facet['label'],
        $response->facet_counts->$type->$field, $options);
    }

    // get date facets
    elseif ($type == 'facet_dates') {
      $links = xc_search_block_get_facet_dates($field, $facet['label'],
        $response->facet_counts->$type->$field, $options);
    } 
    
    // TODO: facet_queries
    elseif ($type == 'facet_queries') {}
    
    // theming links
    if (!empty($links)) {
      $blocks['content'] .= theme('item_list', $links, $facet['label']);
    }
  }
  return $blocks;
}

/**
 * Get the list of field type facets as links
 * @param $field (string)
 *   The name of field
 * @param $field_label (string)
 *   The human readable field name
 * @param $terms (array)
 *   The facet terms. Its items are array or term and count number.
 * @param $options (Array)
 *   An associative array of parameters.
 *     'search_path' - (String) The current path with search term
 *     'filters'     - (Array) Active filters
 *     'non_filters_params' - (Array) Other parameters in URL, which are not filters
 *     'is_prepopulated' - (Boolean) Flag showing whether the facets created 
 *       as prepopulated (without query term), or not
 * @return (Array)
 *   List of links which could filter the current search
 */
function xc_search_block_get_facet_fields($field, $field_label, $terms, $options) {
  foreach ($terms as $term => $count) {
    $fq = $field . ':"' . $term . '"';
    $path = $options['search_path'];
    if ($options['is_prepopulated'] == TRUE) {
      $path .= '/' . $fq;
      $query = array_merge(
        $options['non_filter_params'], 
        array('filter' => $options['filters'])
      );
    }
    else {
      $query = array_merge(
        $options['non_filter_params'], 
        array('filter' => array_merge($options['filters'], array($fq)))
      );
    }
    $fielter_by_label = $field_label . ' = ' . $term;
    $links[] = theme('xc_search_block_facet_link', $term, $count, $path, 
      $fielter_by_label, $query);
  }
  return $links;
}

/**
 * Get the list of date type facets as links
 * @param $field (string)
 *   The name of field
 * @param $field_label (string)
 *   The human readable field name
 * @param $terms (array)
 *   The facet terms. Its items are array or term and count number.
 * @param $options (Array)
 *   An associative array of parameters.
 *     'search_path' - (String) The current path with search term
 *     'filters'     - (Array) Active filters
 *     'non_filters_params' - (Array) Other parameters in URL, which are not filters
 *     'is_prepopulated' - (Boolean) Flag showing whether the facets created 
 *       as prepopulated (without query term), or not
 * @return (Array)
 *   List of links which could filter the current search
 */
function xc_search_block_get_facet_dates($field, $field_label, $terms, $options) {
  static $unhandled_values;

  if(empty($unhandled_values)) {
    $unhandled_values = array('gap', 'end', 'before', 'after', 'between');
  }
  
  $date_filters = xc_search_remove_date_filter($options['filters'], $field);
  $keys = array_keys(get_object_vars($terms));
  
  // parsing gap
  $gap = xc_solr_parse_gap($terms->gap);
  
  // iterate over terms
  for ($i=0; $i<count($keys); $i++) {
    $term  = $keys[$i];
    $count = $terms->$term;
    
    if (in_array($term, $unhandled_values)) {
      continue;
    }
    // there is no terms
    if ($count == 0) {
      continue;
    }
    if (preg_match('/^\d{4}-/', $keys[$i+1])) {
      $next = $keys[$i+1]; //xc_solr_date_modify($keys[$i+1], '-1 second');
    }
    else {
      $next = $terms->end;
    }
    if(!is_null($gap)) {
      if($gap->date_part_length > 0) {
        $first_date  = substr($term,  0, $gap->date_part_length);
        $second_date = substr($next, 0, $gap->date_part_length);
        $term_label = ($first_date != $second_date)
          ? $first_date . '-' . $second_date
          : $first_date;
      }
      else {
        $term_label = $term . '-' . $next;
      }
    }
    else {
      $term_label = $term;
    }
    $fq = $field . ':[' . $term . ' TO ' . $next . ']';
    $path = $options['search_path'];
    if ($options['is_prepopulated'] == TRUE) {
      $path .= '/' . urlencode($fq);
      $query = array_merge(
        $options['non_filter_params'],
        array(
          'filter' => $date_filters,
           $field . '_gap' => $terms->gap
        )
      );
    }
    else {
      $query = array_merge(
        $options['non_filter_params'],
        array(
          'filter' => array_merge($date_filters, array($fq)),
          $field . '_gap' => $terms->gap
        )
      );
    }
    $links[] = theme('xc_search_block_facet_link', $term_label, $count, $path, 
      $fq, $query);
  }
  return $links;
}

/**
 * Creating and theming a facet link
 * @param $term (String)
 *   The facet term to display
 * @param $count (int)
 *   The number of documents matching this term
 * @param $path (string)
 *   The query URL
 * @param $filter_by_label
 *   A label in title attribute of the link
 * @param $query (Array)
 *   The query part of the URL
 * @return (String)
 *   The facet link, which filters the content
 */
function theme_xc_search_block_facet_link($term, $count, $path, $filter_by_label, 
    $query) {
  return l(
    $term . ' (' . $count . ')',
    $path,
    array(
      'attributes' => array(
        'title' => t('filter by \'!label\'', array('!label' => $filter_by_label))
      ),
      'query' => $query
    )
  );
}

