<?php

// $Id: xc_solr.info.inc,v 1.1.2.3 2010/05/23 07:52:49 pkiraly Exp $

/**
 * @file
 * More about the Luke calls see http://wiki.apache.org/solr/LukeRequestHandler
 */

/**
 * Display information about the Solr index
 * @return (String)
 *   The tables displaying the state of the index
 */
function xc_solr_info() {
  global $_xc_search_server;
  
  $params = array(
    'search_type' => 'luke',
    'fl' => 'type',
    'numTerms' => 0,
  );
  $response = $_xc_search_server->search(NULL, 0, 0, $params, TRUE);
  $xc_solr_info_last_modified = variable_get('xc_solr_info_last_modified', NULL);
  $xc_solr_info_directory = variable_get('xc_solr_info_directory', NULL);
  if(is_null($xc_solr_info_last_modified) 
      || $response->index->lastModified > $xc_solr_info_last_modified
      || $response->index->directory != $xc_solr_info_directory
      || variable_get('xc_solr_info', '') == ''
      || (isset($_GET['force']) && $_GET['force'] == 1)) {
    $output = xc_sorl_info_get();
  }
  else {
    $output = variable_get('xc_solr_info', '');
  }
  return $output;
}

/**
 * Get the highest and lowest values of a given date type field
 * 
 * highest value: 
 * http://localhost:8983/solr/select?q=*:*&rows=1&fl=date&sort=date%20asc
 * highest value: 
 * http://localhost:8983/solr/select?q=*:*&rows=1&fl=date&sort=date%20desc
 * 
 * @param $date_field (String)
 *   The name of the date field we would like to use
 */
function xc_sorl_info_get_dates($date_field = 'date_df') {
  global $_xc_search_server;
  
  $dates = variable_set('xc_solr_info_dates', array());
  
  $params = array(
    'fl' => $date_field,
    'sort' => $date_field . ' asc',
  );
  $response = $_xc_search_server->search('*:*', 0, 0, $params, TRUE);
  $dates[$date_field]['min'] = $response->response->docs[0]->$date_field;
  
  $params['sort'] = 'date_df desc';
  $response = $_xc_search_server->search('*:*', 0, 0, $params, TRUE);
  $dates[$date_field]['max'] = $response->response->docs[0]->$date_field;
  
  variable_set('xc_solr_info_dates', $dates);
}

/**
 * Get information about the Solr index
 * @return unknown_type
 */
function xc_sorl_info_get() {
  global $_xc_search_server;

  $params = array(
    'search_type' => 'luke',
  );
  $response = $_xc_search_server->search(NULL, 0, 0, $params, TRUE);
  variable_set('xc_solr_info_last_modified', $response->index->lastModified);
  variable_set('xc_solr_info_directory', $response->index->directory);
  $fields = array_keys(get_object_vars($response->fields));
  
  $index_features = array(
    'numDocs' => t('Number of docuemts'),
    'numTerms' => t('Number of terms'),
    'version' => t('Solr version release date'),
    'optimized' => t('Index is optimized?'),
    'current' => t('Current commit user data?'),
    'hasDeletions' => t('Index has deletions?'),
    'lastModified' => t('Last modified'),
  );
  $types = 
  $field_features = array(
    'type' => t('Type'),
    'schema' => t('Indexing properties'),
    'dynamicBase' => t('Field suffix'),
    'docs' => t('The number of documents it appears in'),
    'distinct' => t('The number of distinct values'),
  );
  
  // index features
  $rows = array();
  $rows[] = array(t('Number of fields'), count($fields));
  foreach($index_features as $feature => $label) {
    $value = $response->index->$feature;
    switch($feature) {
      case 'hasDeletions':
      case 'optimized':
      case 'current':
        $value = $value ? t('Yes') : t('No');
        break;
        
      case 'version':
        $value = format_date(intval($value/1000), 'custom', "Y-m-d h:m:s");
        break;
      case 'lastModified':
        $value = str_replace('T', ' ', str_replace('Z', '', $value));
        break;
        
      case 'numDocs':
      case 'numTerms':
        $value = number_format($value, 0, ',', ' ');
        break;
    }
    $rows[] = array($label, $value);
  }
  $output = theme('table', array(t('Feature'), t('Value')), $rows, array(), 
    t('The state of Solr index'));
  
  // field features
  $rows = array();
  sort($fields);
  foreach($fields as $field) {

    $features = array();
    foreach($field_features as $feature => $label) {
      if(isset($response->fields->$field->$feature)) {
        $value = $response->fields->$field->$feature;
        switch($feature) {
          case 'docs':
          case 'distinct':
            $value = number_format($value, 0, ',', ' ');
            break;
        }
        $features[] = array(
          'data' => array(array('data' => $label, 'width' => '25%'), $value),
          'valign' => 'top'
        );
      }
    }
    if(isset($response->fields->$field->topTerms)) {
      $top_terms = array();
      foreach($response->fields->$field->topTerms as $term => $count) {
        switch($response->fields->$field->type) {
          case 'string':
          case 'date':
            $query = $field . ':"' . $term . '"';
            break;
          default:
            $query = $field . ':(' . $term . ')';
        }
        $top_terms[] = sprintf("%s (%d)", 
          l($term, 'search/xc_search/' . $query, array('query' => 'debug=1')), 
          number_format($count, 0, ',', ' ')
        );
      };
      if(!empty($top_terms)) {
        $features[] =  array(
          'data' => array(t('Top terms'), theme('item_list', $top_terms)),
          'valign' => 'top'
        );
      }
    }
    $field_info = theme('table', array(t('Feature'), t('Value')), $features);
    $rows[] = array(
      'data' => array(array('data' => $field, 'width' => '25%'), $field_info),
      'valign' => 'top'
    );
  }
  $output .= theme('table', array(t('Field name'), t('Features')), $rows, 
    array(), t('Properties of fields'));
  variable_set('xc_solr_info', $output);
  
  return $output;
}